﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;

namespace HeuristicLab.Services.ProblemInstances {
  public class QAP : IQAP {
    public string[] GetProblemInstances() {
      using (ProblemInstancesEntities context = new ProblemInstancesEntities()) {
        return context.QAPInstances.Select(x => x.Name).ToArray();
      }
    }

    public QAPInstanceDto GetProblemInstanceData(string instance) {
      using (ProblemInstancesEntities context = new ProblemInstancesEntities()) {
        return new QAPInstanceDto(context.QAPInstances.Where(x => x.Name == instance).Single());
      }
    }

    public QAPSolutionDto[] GetBestSolutionsData(string instance) {
      using (ProblemInstancesEntities context = new ProblemInstancesEntities()) {
        QAPInstance qap = context.QAPInstances.Where(x => x.Name == instance).SingleOrDefault();
        if (qap == null) return new QAPSolutionDto[0];
        context.LoadProperty<QAPInstance>(qap, x => x.Solutions);
        double best = double.NaN;
        if (!qap.Solutions.Any()) return new QAPSolutionDto[0];

        if (qap.Maximization) best = qap.Solutions.Max(x => x.Quality);
        else best = qap.Solutions.Min(x => x.Quality);

        return qap.Solutions.Where(x => x.Quality == best).Select(x => new QAPSolutionDto(x)).ToArray();
      }
    }
  }
}
