﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using System.Runtime.Serialization;

namespace HeuristicLab.Services.ProblemInstances {
  [DataContract]
  public class QAPInstanceDto {
    [DataMember]
    public string Name { get; set; }
    [DataMember]
    public string Description { get; set; }
    [DataMember]
    public bool Maximization { get; set; }
    [DataMember]
    public double[][] Weights { get; set; }
    [DataMember]
    public double[][] Distances { get; set; }
    [DataMember]
    public double BestKnownQuality { get; set; }
    [DataMember]
    public bool HasBestKnownQuality { get; set; }

    public QAPInstanceDto() {
      Name = "-=UNDEFINED=-";
    }

    public QAPInstanceDto(QAPInstance qAPInstance) {
      Name = qAPInstance.Name;
      Description = qAPInstance.Description;
      Maximization = qAPInstance.Maximization;
      try {
        Weights = SimpleSerializer.DeserializeDoubleMatrix(qAPInstance.Weights);
        Distances = SimpleSerializer.DeserializeDoubleMatrix(qAPInstance.Distances);
      } catch (ArgumentException e) {
        throw new SimpleSerializationException("Failed to deserialize the weights or distance matrix.", e);
      }
      BestKnownQuality = double.NaN;
      HasBestKnownQuality = false;
      if (qAPInstance.Solutions.Any()) {
        double bestQuality;
        if (Maximization) bestQuality = qAPInstance.Solutions.Max(x => x.Quality);
        else bestQuality = qAPInstance.Solutions.Min(x => x.Quality);
        HasBestKnownQuality = true;
      }
    }

    public bool Validate() {
      bool valid = Name.Trim() != string.Empty
        && Name != "-=UNDEFINED=-"
        && Weights.GetLength(0) == Distances.GetLength(0);
      if (valid) {
        for (int i = 0; i < Weights.GetLength(0); i++) {
          valid = Weights[i].Length == Distances[i].Length;
          if (!valid) break;
        }
      }
      return valid;
    }
  }
}
